@@ -32,6 +32,8 @@
 #define LTTNG_COMPACT_EVENT_BITS	5
 #define LTTNG_COMPACT_TSC_BITS		27
 
+static struct lttng_transport lttng_relay_transport;
+
 /*
  * Keep the natural field alignment for _each field_ within this structure if
  * you ever add/remove a field from this header. Packed attribute is not used
@@ -60,13 +62,13 @@ struct packet_header {
 		/* Stream packet context */
 		uint64_t timestamp_begin;	/* Cycle count at subbuffer start */
 		uint64_t timestamp_end;		/* Cycle count at subbuffer end */
-		uint32_t events_discarded;	/*
+		uint64_t content_size;		/* Size of data in subbuffer */
+		uint64_t packet_size;		/* Subbuffer size (include padding) */
+		unsigned long events_discarded;	/*
 						 * Events lost in this subbuffer since
 						 * the beginning of the trace.
 						 * (may overflow)
 						 */
-		uint32_t content_size;		/* Size of data in subbuffer */
-		uint32_t packet_size;		/* Subbuffer size (include padding) */
 		uint32_t cpu_id;		/* CPU id associated with stream */
 		uint8_t header_end;		/* End of header */
 	} ctx;
@@ -349,9 +351,9 @@ static void client_buffer_begin(struct lib_ring_buffer *buf, u64 tsc,
 	header->stream_id = lttng_chan->id;
 	header->ctx.timestamp_begin = tsc;
 	header->ctx.timestamp_end = 0;
+	header->ctx.content_size = ~0ULL; /* for debugging */
+	header->ctx.packet_size = ~0ULL;
 	header->ctx.events_discarded = 0;
-	header->ctx.content_size = 0xFFFFFFFF; /* for debugging */
-	header->ctx.packet_size = 0xFFFFFFFF;
 	header->ctx.cpu_id = buf->backend.cpu;
 }
 
@@ -370,8 +372,10 @@ static void client_buffer_end(struct lib_ring_buffer *buf, u64 tsc,
 	unsigned long records_lost = 0;
 
 	header->ctx.timestamp_end = tsc;
-	header->ctx.content_size = data_size * CHAR_BIT; 	/* in bits */
-	header->ctx.packet_size = PAGE_ALIGN(data_size) * CHAR_BIT; /* in bits */
+	header->ctx.content_size =
+		(uint64_t) data_size * CHAR_BIT;		/* in bits */
+	header->ctx.packet_size =
+		(uint64_t) PAGE_ALIGN(data_size) * CHAR_BIT;	/* in bits */
 	records_lost += lib_ring_buffer_get_records_lost_full(&client_config, buf);
 	records_lost += lib_ring_buffer_get_records_lost_wrap(&client_config, buf);
 	records_lost += lib_ring_buffer_get_records_lost_big(&client_config, buf);
@@ -388,6 +392,82 @@ static void client_buffer_finalize(struct lib_ring_buffer *buf, void *priv, int
 {
 }
 
+static struct packet_header *client_packet_header(
+		const struct lib_ring_buffer_config *config,
+		struct lib_ring_buffer *buf)
+{
+	return lib_ring_buffer_read_offset_address(&buf->backend, 0);
+}
+
+static int client_timestamp_begin(const struct lib_ring_buffer_config *config,
+		struct lib_ring_buffer *buf,
+		uint64_t *timestamp_begin)
+{
+	struct packet_header *header = client_packet_header(config, buf);
+	*timestamp_begin = header->ctx.timestamp_begin;
+
+	return 0;
+}
+
+static int client_timestamp_end(const struct lib_ring_buffer_config *config,
+			struct lib_ring_buffer *buf,
+			uint64_t *timestamp_end)
+{
+	struct packet_header *header = client_packet_header(config, buf);
+	*timestamp_end = header->ctx.timestamp_end;
+
+	return 0;
+}
+
+static int client_events_discarded(const struct lib_ring_buffer_config *config,
+			struct lib_ring_buffer *buf,
+			uint64_t *events_discarded)
+{
+	struct packet_header *header = client_packet_header(config, buf);
+	*events_discarded = header->ctx.events_discarded;
+
+	return 0;
+}
+
+static int client_content_size(const struct lib_ring_buffer_config *config,
+			struct lib_ring_buffer *buf,
+			uint64_t *content_size)
+{
+	struct packet_header *header = client_packet_header(config, buf);
+	*content_size = header->ctx.content_size;
+
+	return 0;
+}
+
+static int client_packet_size(const struct lib_ring_buffer_config *config,
+			struct lib_ring_buffer *buf,
+			uint64_t *packet_size)
+{
+	struct packet_header *header = client_packet_header(config, buf);
+	*packet_size = header->ctx.packet_size;
+
+	return 0;
+}
+
+static int client_stream_id(const struct lib_ring_buffer_config *config,
+			struct lib_ring_buffer *buf,
+			uint64_t *stream_id)
+{
+	struct packet_header *header = client_packet_header(config, buf);
+	*stream_id = header->stream_id;
+
+	return 0;
+}
+
+static int client_current_timestamp(const struct lib_ring_buffer_config *config,
+		struct lib_ring_buffer *bufb,
+		uint64_t *ts)
+{
+	*ts = config->cb.ring_buffer_clock_read(bufb->backend.chan);
+
+	return 0;
+}
+
 static const struct lib_ring_buffer_config client_config = {
 	.cb.ring_buffer_clock_read = client_ring_buffer_clock_read,
 	.cb.record_header_size = client_record_header_size,
@@ -409,15 +489,9 @@ static const struct lib_ring_buffer_config client_config = {
 };
 
 static
-struct channel *_channel_create(const char *name,
-				struct lttng_channel *lttng_chan, void *buf_addr,
-				size_t subbuf_size, size_t num_subbuf,
-				unsigned int switch_timer_interval,
-				unsigned int read_timer_interval)
+void release_priv_ops(void *priv_ops)
 {
-	return channel_create(&client_config, name, lttng_chan, buf_addr,
-			      subbuf_size, num_subbuf, switch_timer_interval,
-			      read_timer_interval);
+	module_put(THIS_MODULE);
 }
 
 static
@@ -426,6 +500,37 @@ void lttng_channel_destroy(struct channel *chan)
 	channel_destroy(chan);
 }
 
+static
+struct channel *_channel_create(const char *name,
+				struct lttng_channel *lttng_chan, void *buf_addr,
+				size_t subbuf_size, size_t num_subbuf,
+				unsigned int switch_timer_interval,
+				unsigned int read_timer_interval)
+{
+	struct channel *chan;
+
+	chan = channel_create(&client_config, name, lttng_chan, buf_addr,
+			      subbuf_size, num_subbuf, switch_timer_interval,
+			      read_timer_interval);
+	if (chan) {
+		/*
+		 * Ensure this module is not unloaded before we finish
+		 * using lttng_relay_transport.ops.
+		 */
+		if (!try_module_get(THIS_MODULE)) {
+			printk(KERN_WARNING "LTT : Can't lock transport module.\n");
+			goto error;
+		}
+		chan->backend.priv_ops = &lttng_relay_transport.ops;
+		chan->backend.release_priv_ops = release_priv_ops;
+	}
+	return chan;
+
+error:
+	lttng_channel_destroy(chan);
+	return NULL;
+}
+
 static
 struct lib_ring_buffer *lttng_buffer_read_open(struct channel *chan)
 {
@@ -513,7 +618,7 @@ static
 void lttng_event_write_from_user(struct lib_ring_buffer_ctx *ctx,
 			       const void __user *src, size_t len)
 {
-	lib_ring_buffer_copy_from_user(&client_config, ctx, src, len);
+	lib_ring_buffer_copy_from_user_inatomic(&client_config, ctx, src, len);
 }
 
 static
@@ -569,6 +674,13 @@ static struct lttng_transport lttng_relay_transport = {
 		.get_hp_wait_queue = lttng_get_hp_wait_queue,
 		.is_finalized = lttng_is_finalized,
 		.is_disabled = lttng_is_disabled,
+		.timestamp_begin = client_timestamp_begin,
+		.timestamp_end = client_timestamp_end,
+		.events_discarded = client_events_discarded,
+		.content_size = client_content_size,
+		.packet_size = client_packet_size,
+		.stream_id = client_stream_id,
+		.current_timestamp = client_current_timestamp,
 	},
 };
 
